@@ -17,20 +17,24 @@ import os
 
 import ddt
 import mock
-from oslotest import base
 import six
 
 from os_win import constants
 from os_win import exceptions
+from os_win.tests.unit import test_base
 from os_win.utils.storage.virtdisk import vhdutils
 from os_win.utils.winapi import constants as w_const
 from os_win.utils.winapi import wintypes
 
 
 @ddt.ddt
-class VHDUtilsTestCase(base.BaseTestCase):
+class VHDUtilsTestCase(test_base.BaseTestCase):
     """Unit tests for the Hyper-V VHDUtils class."""
 
+    _autospec_classes = [
+        vhdutils.win32utils.Win32Utils,
+    ]
+
     def setUp(self):
         super(VHDUtilsTestCase, self).setUp()
         self._setup_lib_mocks()
@@ -38,7 +42,6 @@ class VHDUtilsTestCase(base.BaseTestCase):
         self._fake_vst_struct = self._vdisk_struct.VIRTUAL_STORAGE_TYPE
 
         self._vhdutils = vhdutils.VHDUtils()
-        self._vhdutils._win32_utils = mock.Mock()
 
         self._mock_close = self._vhdutils._win32_utils.close_handle
         self._mock_run = self._vhdutils._win32_utils.run_and_check_output
@@ -814,23 +817,30 @@ class VHDUtilsTestCase(base.BaseTestCase):
             bool(attach_flag &
                  w_const.ATTACH_VIRTUAL_DISK_FLAG_PERMANENT_LIFETIME))
 
+    @ddt.data(True, False)
+    @mock.patch('os.path.exists')
     @mock.patch.object(vhdutils.VHDUtils, '_open')
-    def test_detach_virtual_disk(self, mock_open):
+    def test_detach_virtual_disk(self, exists, mock_open, mock_exists):
+        mock_exists.return_value = exists
         self._mock_run.return_value = w_const.ERROR_NOT_READY
 
         self._vhdutils.detach_virtual_disk(mock.sentinel.vhd_path)
 
-        mock_open.assert_called_once_with(
-            mock.sentinel.vhd_path,
-            open_access_mask=w_const.VIRTUAL_DISK_ACCESS_DETACH)
+        mock_exists.assert_called_once_with(mock.sentinel.vhd_path)
+        if exists:
+            mock_open.assert_called_once_with(
+                mock.sentinel.vhd_path,
+                open_access_mask=w_const.VIRTUAL_DISK_ACCESS_DETACH)
 
-        self._mock_run.assert_called_once_with(
-            vhdutils.virtdisk.DetachVirtualDisk,
-            mock_open.return_value,
-            0, 0,
-            ignored_error_codes=[w_const.ERROR_NOT_READY],
-            **self._run_args)
-        self._mock_close.assert_called_once_with(mock_open.return_value)
+            self._mock_run.assert_called_once_with(
+                vhdutils.virtdisk.DetachVirtualDisk,
+                mock_open.return_value,
+                0, 0,
+                ignored_error_codes=[w_const.ERROR_NOT_READY],
+                **self._run_args)
+            self._mock_close.assert_called_once_with(mock_open.return_value)
+        else:
+            mock_open.assert_not_called()
 
     @mock.patch.object(vhdutils.VHDUtils, '_open')
     def test_get_virtual_disk_physical_path(self, mock_open):
