@@ -49,7 +49,6 @@ class ClusterUtils(baseutils.BaseUtils):
 
     _VM_BASE_NAME = 'Virtual Machine %s'
     _VM_TYPE = 'Virtual Machine'
-    _VM_GROUP_TYPE = 111
 
     _MS_CLUSTER_NAMESPACE = '//%s/root/MSCluster'
 
@@ -66,10 +65,10 @@ class ClusterUtils(baseutils.BaseUtils):
     def __init__(self, host='.'):
         self._instance_name_regex = re.compile('Virtual Machine (.*)')
         self._clusapi_utils = _clusapi_utils.ClusApiUtils()
+        self._cmgr = _clusapi_utils.ClusterContextManager()
 
         if sys.platform == 'win32':
             self._init_hyperv_conn(host)
-            self._watcher = self._get_failover_watcher()
 
     def _init_hyperv_conn(self, host):
         try:
@@ -97,7 +96,7 @@ class ClusterUtils(baseutils.BaseUtils):
         return self._conn_cluster.watch_for(raw_wql=raw_query)
 
     def check_cluster_state(self):
-        if len(self._get_cluster_nodes()) < 1:
+        if len(list(self._get_cluster_nodes())) < 1:
             raise exceptions.HyperVClusterException(
                 _("Not enough cluster nodes."))
 
@@ -105,17 +104,13 @@ class ClusterUtils(baseutils.BaseUtils):
         return self._this_node
 
     def _get_cluster_nodes(self):
-        cluster_assoc = self._conn_cluster.MSCluster_ClusterToNode(
-            Antecedent=self._cluster.path_())
-        return [x.Dependent for x in cluster_assoc]
+        return self.cluster_enum(w_const.CLUSTER_ENUM_NODE)
 
     def _get_vm_groups(self):
-        assocs = self._conn_cluster.MSCluster_ClusterToResourceGroup(
-            GroupComponent=self._cluster.path_())
-        resources = [a.PartComponent for a in assocs]
-        return (r for r in resources if
-                hasattr(r, 'GroupType') and
-                r.GroupType == self._VM_GROUP_TYPE)
+        for r in self.cluster_enum(w_const.CLUSTER_ENUM_GROUP):
+            group_type = self.get_cluster_group_type(r['name'])
+            if group_type == w_const.ClusGroupTypeVirtualMachine:
+                yield r
 
     def _lookup_vm_group_check(self, vm_name):
         vm = self._lookup_vm_group(vm_name)
@@ -127,16 +122,6 @@ class ClusterUtils(baseutils.BaseUtils):
         return self._lookup_res(self._conn_cluster.MSCluster_ResourceGroup,
                                 vm_name)
 
-    def _lookup_vm_check(self, vm_name):
-        vm = self._lookup_vm(vm_name)
-        if not vm:
-            raise exceptions.HyperVVMNotFoundException(vm_name=vm_name)
-        return vm
-
-    def _lookup_vm(self, vm_name):
-        vm_name = self._VM_BASE_NAME % vm_name
-        return self._lookup_res(self._conn_cluster.MSCluster_Resource, vm_name)
-
     def _lookup_res(self, resource_source, res_name):
         res = resource_source(Name=res_name)
         n = len(res)
@@ -150,16 +135,18 @@ class ClusterUtils(baseutils.BaseUtils):
 
     def get_cluster_node_names(self):
         nodes = self._get_cluster_nodes()
-        return [n.Name for n in nodes]
+        return [n['name'] for n in nodes]
 
     def get_vm_host(self, vm_name):
-        return self._lookup_vm_group_check(vm_name).OwnerNode
+        with self._cmgr.open_cluster_group(vm_name) as group_handle:
+            state_info = self._get_cluster_group_state(group_handle)
+            return state_info['owner_node']
 
     def list_instances(self):
-        return [r.Name for r in self._get_vm_groups()]
+        return [r['name'] for r in self._get_vm_groups()]
 
     def list_instance_uuids(self):
-        return [r.Id for r in self._get_vm_groups()]
+        return [r['id'] for r in self._get_vm_groups()]
 
     def add_vm_to_cluster(self, vm_name, max_failover_count=1,
                           failover_period=6, auto_failback=True):
@@ -191,19 +178,41 @@ class ClusterUtils(baseutils.BaseUtils):
         vm_group.put()
 
     def bring_online(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        vm.BringOnline()
+        with self._cmgr.open_cluster_group(vm_name) as group_handle:
+            self._clusapi_utils.online_cluster_group(group_handle)
 
     def take_offline(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        vm.TakeOffline()
+        with self._cmgr.open_cluster_group(vm_name) as group_handle:
+            self._clusapi_utils.offline_cluster_group(group_handle)
 
     def delete(self, vm_name):
+        # We're sticking with WMI, for now. Destroying VM cluster groups using
+        # clusapi's DestroyClusterGroup function acts strange. VMs get
+        # recreated asyncronuously and put in suspended state,
+        # breaking everything.
         vm = self._lookup_vm_group_check(vm_name)
         vm.DestroyGroup(self._DESTROY_GROUP)
 
+    def cluster_enum(self, object_type):
+        with self._cmgr.open_cluster_enum(object_type) as enum_handle:
+            object_count = self._clusapi_utils.cluster_get_enum_count(
+                enum_handle)
+            for idx in range(object_count):
+                item = self._clusapi_utils.cluster_enum(enum_handle, idx)
+
+                item_dict = dict(version=item.dwVersion,
+                                 type=item.dwType,
+                                 id=item.lpszId,
+                                 name=item.lpszName)
+                yield item_dict
+
     def vm_exists(self, vm_name):
-        return self._lookup_vm(vm_name) is not None
+        res_name = self._VM_BASE_NAME % vm_name
+        try:
+            with self._cmgr.open_cluster_resource(res_name):
+                return True
+        except exceptions.ClusterObjectNotFound:
+            return False
 
     def live_migrate_vm(self, vm_name, new_host, timeout=None):
         self._migrate_vm(vm_name, new_host, self._LIVE_MIGRATION_TYPE,
@@ -228,62 +237,51 @@ class ClusterUtils(baseutils.BaseUtils):
             w_const.CLUSAPI_GROUP_MOVE_QUEUE_ENABLED |
             w_const.CLUSAPI_GROUP_MOVE_HIGH_PRIORITY_START)
 
-        cluster_handle = None
-        group_handle = None
-        dest_node_handle = None
+        with self._cmgr.open_cluster() as cluster_handle, \
+                self._cmgr.open_cluster_group(
+                    vm_name,
+                    cluster_handle=cluster_handle) as group_handle, \
+                self._cmgr.open_cluster_node(
+                    new_host,
+                    cluster_handle=cluster_handle) as dest_node_handle, \
+                _ClusterGroupStateChangeListener(cluster_handle,
+                                                 vm_name) as listener:
+            self._clusapi_utils.move_cluster_group(group_handle,
+                                                   dest_node_handle,
+                                                   flags,
+                                                   prop_list)
+            try:
+                self._wait_for_cluster_group_migration(
+                    listener,
+                    vm_name,
+                    group_handle,
+                    exp_state_after_migr,
+                    timeout)
+            except exceptions.ClusterGroupMigrationTimeOut:
+                with excutils.save_and_reraise_exception() as ctxt:
+                    self._cancel_cluster_group_migration(
+                        listener, vm_name, group_handle,
+                        exp_state_after_migr, timeout)
 
-        try:
-            cluster_handle = self._clusapi_utils.open_cluster()
-            group_handle = self._clusapi_utils.open_cluster_group(
-                cluster_handle, vm_name)
-            dest_node_handle = self._clusapi_utils.open_cluster_node(
-                cluster_handle, new_host)
-
-            with _ClusterGroupStateChangeListener(cluster_handle,
-                                                  vm_name) as listener:
-                self._clusapi_utils.move_cluster_group(group_handle,
-                                                       dest_node_handle,
-                                                       flags,
-                                                       prop_list)
-                try:
-                    self._wait_for_cluster_group_migration(
-                        listener,
-                        vm_name,
-                        group_handle,
-                        exp_state_after_migr,
-                        timeout)
-                except exceptions.ClusterGroupMigrationTimeOut:
-                    with excutils.save_and_reraise_exception() as ctxt:
-                        self._cancel_cluster_group_migration(
-                            listener, vm_name, group_handle,
-                            exp_state_after_migr, timeout)
-
-                        # This is rather unlikely to happen but we're
-                        # covering it out.
-                        try:
-                            self._validate_migration(group_handle,
-                                                     vm_name,
-                                                     exp_state_after_migr,
-                                                     new_host)
-                            LOG.warning(
-                                'Cluster group migration completed '
-                                'successfuly after cancel attempt. '
-                                'Suppressing timeout exception.')
-                            ctxt.reraise = False
-                        except exceptions.ClusterGroupMigrationFailed:
-                            pass
-                else:
-                    self._validate_migration(group_handle,
-                                             vm_name,
-                                             exp_state_after_migr,
-                                             new_host)
-        finally:
-            if group_handle:
-                self._clusapi_utils.close_cluster_group(group_handle)
-            if dest_node_handle:
-                self._clusapi_utils.close_cluster_node(dest_node_handle)
-            if cluster_handle:
-                self._clusapi_utils.close_cluster(cluster_handle)
+                    # This is rather unlikely to happen but we're
+                    # covering it out.
+                    try:
+                        self._validate_migration(group_handle,
+                                                 vm_name,
+                                                 exp_state_after_migr,
+                                                 new_host)
+                        LOG.warning(
+                            'Cluster group migration completed '
+                            'successfuly after cancel attempt. '
+                            'Suppressing timeout exception.')
+                        ctxt.reraise = False
+                    except exceptions.ClusterGroupMigrationFailed:
+                        pass
+            else:
+                self._validate_migration(group_handle,
+                                         vm_name,
+                                         exp_state_after_migr,
+                                         new_host)
 
     def _validate_migration(self, group_handle, group_name,
                             expected_state, expected_node):
@@ -302,24 +300,15 @@ class ClusterUtils(baseutils.BaseUtils):
 
     def cancel_cluster_group_migration(self, group_name, expected_state,
                                        timeout=None):
-        cluster_handle = None
-        group_handle = None
-
-        try:
-            cluster_handle = self._clusapi_utils.open_cluster()
-            group_handle = self._clusapi_utils.open_cluster_group(
-                cluster_handle, group_name)
-
-            with _ClusterGroupStateChangeListener(cluster_handle,
-                                                  group_name) as listener:
-                self._cancel_cluster_group_migration(
-                    listener, group_name, group_handle,
-                    expected_state, timeout)
-        finally:
-            if group_handle:
-                self._clusapi_utils.close_cluster_group(group_handle)
-            if cluster_handle:
-                self._clusapi_utils.close_cluster(cluster_handle)
+        with self._cmgr.open_cluster() as cluster_handle, \
+                self._cmgr.open_cluster_group(
+                    group_name,
+                    cluster_handle=cluster_handle) as group_handle, \
+                _ClusterGroupStateChangeListener(cluster_handle,
+                                                 group_name) as listener:
+            self._cancel_cluster_group_migration(
+                listener, group_name, group_handle,
+                expected_state, timeout)
 
     def _cancel_cluster_group_migration(self, event_listener,
                                         group_name, group_handle,
@@ -417,20 +406,28 @@ class ClusterUtils(baseutils.BaseUtils):
             group_name=group_name,
             time_elapsed=time.time() - time_start)
 
+    def get_cluster_node_name(self, node_id):
+        for node in self._get_cluster_nodes():
+            if node['id'] == node_id:
+                return node['name']
+
+        err_msg = _("Could not find any cluster node with id: %s.")
+        raise exceptions.NotFound(err_msg % node_id)
+
+    def get_cluster_group_type(self, group_name):
+        with self._cmgr.open_cluster_group(group_name) as group_handle:
+            buff, buff_sz = self._clusapi_utils.cluster_group_control(
+                group_handle, w_const.CLUSCTL_GROUP_GET_RO_COMMON_PROPERTIES)
+            return self._clusapi_utils.get_cluster_group_type(
+                ctypes.byref(buff), buff_sz)
+
     def get_cluster_group_state_info(self, group_name):
         """Gets cluster group state info.
 
         :return: a dict containing the following keys:
             ['state', 'migration_queued', 'owner_node']
         """
-        cluster_handle = None
-        group_handle = None
-
-        try:
-            cluster_handle = self._clusapi_utils.open_cluster()
-            group_handle = self._clusapi_utils.open_cluster_group(
-                cluster_handle, group_name)
-
+        with self._cmgr.open_cluster_group(group_name) as group_handle:
             state_info = self._get_cluster_group_state(group_handle)
             migration_queued = self._is_migration_queued(
                 state_info['status_info'])
@@ -438,11 +435,6 @@ class ClusterUtils(baseutils.BaseUtils):
             return dict(owner_node=state_info['owner_node'],
                         state=state_info['state'],
                         migration_queued=migration_queued)
-        finally:
-            if group_handle:
-                self._clusapi_utils.close_cluster_group(group_handle)
-            if cluster_handle:
-                self._clusapi_utils.close_cluster(cluster_handle)
 
     def _get_cluster_group_state(self, group_handle):
         state_info = self._clusapi_utils.get_cluster_group_state(group_handle)
@@ -456,8 +448,8 @@ class ClusterUtils(baseutils.BaseUtils):
         state_info['status_info'] = status_info
         return state_info
 
-    def monitor_vm_failover(self, callback,
-                            event_timeout_ms=_WMI_EVENT_TIMEOUT_MS):
+    def _monitor_vm_failover(self, watcher, callback,
+                             event_timeout_ms=_WMI_EVENT_TIMEOUT_MS):
         """Creates a monitor to check for new WMI MSCluster_Resource
 
         events.
@@ -469,19 +461,15 @@ class ClusterUtils(baseutils.BaseUtils):
         Any event object caught will then be processed.
         """
 
-        # TODO(lpetrut): mark this method as private once compute-hyperv
-        # stops using it. We should also remove the instance '_watcher'
-        # attribute since we end up spawning unused event listeners.
-
         vm_name = None
         new_host = None
         try:
             # wait for new event for _WMI_EVENT_TIMEOUT_MS milliseconds.
             if patcher.is_monkey_patched('thread'):
-                wmi_object = tpool.execute(self._watcher,
+                wmi_object = tpool.execute(watcher,
                                            event_timeout_ms)
             else:
-                wmi_object = self._watcher(event_timeout_ms)
+                wmi_object = watcher(event_timeout_ms)
 
             old_host = wmi_object.previous.OwnerNode
             new_host = wmi_object.OwnerNode
@@ -504,13 +492,16 @@ class ClusterUtils(baseutils.BaseUtils):
 
     def get_vm_owner_change_listener(self):
         def listener(callback):
+            watcher = self._get_failover_watcher()
+
             while True:
                 # We avoid setting an infinite timeout in order to let
                 # the process gracefully stop. Note that the os-win WMI
                 # event listeners are meant to be used as long running
                 # daemons, so no stop API is provided ATM.
                 try:
-                    self.monitor_vm_failover(
+                    self._monitor_vm_failover(
+                        watcher,
                         callback,
                         constants.DEFAULT_WMI_EVENT_TIMEOUT_MS)
                 except Exception:
@@ -521,6 +512,30 @@ class ClusterUtils(baseutils.BaseUtils):
 
         return listener
 
+    def get_vm_owner_change_listener_v2(self):
+        def listener(callback):
+            cluster_handle = self._clusapi_utils.open_cluster()
+            _listener = _ClusterGroupOwnerChangeListener(cluster_handle)
+
+            while True:
+                try:
+                    event = _listener.get()
+                    group_name = event['cluster_object_name']
+                    group_type = self.get_cluster_group_type(group_name)
+                    if group_type != w_const.ClusGroupTypeVirtualMachine:
+                        continue
+
+                    new_node_id = event['parent_id']
+                    new_node_name = self.get_cluster_node_name(new_node_id)
+                    callback(group_name, new_node_name)
+                except Exception:
+                    LOG.exception("The VM cluster group owner change "
+                                  "event listener encountered an "
+                                  "unexpected exception.")
+                    time.sleep(constants.DEFAULT_WMI_EVENT_TIMEOUT_MS / 1000)
+
+        return listener
+
 
 # At the moment, those event listeners are not meant to be used outside
 # os-win, mostly because of the underlying API limitations.
@@ -529,10 +544,12 @@ class _ClusterEventListener(object):
     _notif_port_h = None
     _cluster_handle = None
     _running = False
+    _stop_on_error = True
+    _error_sleep_interval = 2
 
-    def __init__(self, cluster_handle, notif_filters_list):
+    def __init__(self, cluster_handle, stop_on_error=True):
         self._cluster_handle = cluster_handle
-        self._notif_filters_list = notif_filters_list
+        self._stop_on_error = stop_on_error
 
         self._clusapi_utils = _clusapi_utils.ClusApiUtils()
         self._event_queue = queue.Queue()
@@ -609,9 +626,13 @@ class _ClusterEventListener(object):
             except Exception:
                 if self._running:
                     LOG.exception(
-                        "Unexpected exception in event listener loop. "
-                        "The cluster event listener will now close.")
-                    self._signal_stopped()
+                        "Unexpected exception in event listener loop.")
+                    if self._stop_on_error:
+                        LOG.warning(
+                            "The cluster event listener will now close.")
+                        self._signal_stopped()
+                    else:
+                        time.sleep(self._error_sleep_interval)
 
     def _process_event(self, event):
         return event
@@ -642,11 +663,11 @@ class _ClusterGroupStateChangeListener(_ClusterEventListener):
              filter_flags=w_const.CLUSTER_CHANGE_GROUP_COMMON_PROPERTY_V2,
              notif_key=_NOTIF_KEY_GROUP_COMMON_PROP)]
 
-    def __init__(self, cluster_handle, group_name=None):
+    def __init__(self, cluster_handle, group_name=None, **kwargs):
         self._group_name = group_name
 
         super(_ClusterGroupStateChangeListener, self).__init__(
-            cluster_handle, self._notif_filters_list)
+            cluster_handle, **kwargs)
 
     def _process_event(self, event):
         group_name = event['cluster_object_name']
@@ -676,3 +697,10 @@ class _ClusterGroupStateChangeListener(_ClusterEventListener):
                 # At the moment, we only care about the 'StatusInformation'
                 # common property.
                 pass
+
+
+class _ClusterGroupOwnerChangeListener(_ClusterEventListener):
+    _notif_filters_list = [
+        dict(object_type=w_const.CLUSTER_OBJECT_TYPE_GROUP,
+             filter_flags=w_const.CLUSTER_CHANGE_GROUP_OWNER_NODE_V2)
+    ]
