@@ -28,6 +28,32 @@ ERROR_MORE_DATA = 234
 ERROR_WAIT_TIMEOUT = 258
 ERROR_IO_PENDING = 997
 ERROR_NOT_FOUND = 1168
+
+# Cluster errors
+ERROR_DEPENDENCY_NOT_FOUND = 5002
+ERROR_RESOURCE_NOT_FOUND = 5007
+ERROR_GROUP_NOT_FOUND = 5013
+ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND = 5032
+ERROR_CLUSTER_NODE_NOT_FOUND = 5042
+ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND = 5043
+ERROR_CLUSTER_NETWORK_NOT_FOUND = 5045
+ERROR_CLUSTER_NETINTERFACE_NOT_FOUND = 5047
+ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND = 5078
+ERROR_CLUSTER_RESNAME_NOT_FOUND = 5080
+ERROR_QUORUM_DISK_NOT_FOUND = 5086
+ERROR_CLUSTER_QUORUMLOG_NOT_FOUND = 5891
+ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP = 5894
+
+CLUSTER_NOT_FOUND_ERROR_CODES = [
+    ERROR_DEPENDENCY_NOT_FOUND, ERROR_RESOURCE_NOT_FOUND,
+    ERROR_GROUP_NOT_FOUND, ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND,
+    ERROR_CLUSTER_NODE_NOT_FOUND, ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND,
+    ERROR_CLUSTER_NETWORK_NOT_FOUND, ERROR_CLUSTER_NETINTERFACE_NOT_FOUND,
+    ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND, ERROR_CLUSTER_RESNAME_NOT_FOUND,
+    ERROR_QUORUM_DISK_NOT_FOUND, ERROR_CLUSTER_QUORUMLOG_NOT_FOUND,
+    ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP,
+]
+
 ERROR_INVALID_STATE = 5023
 ERROR_VHD_INVALID_TYPE = 0xC03A001B
 
@@ -107,16 +133,30 @@ CLUSTER_OBJECT_TYPE_GROUP = 2
 
 CLUSTER_CHANGE_GROUP_COMMON_PROPERTY_V2 = 2
 CLUSTER_CHANGE_GROUP_STATE_V2 = 8
+CLUSTER_CHANGE_GROUP_OWNER_NODE_V2 = 0x00000010
 
 CLUSGRP_STATUS_WAITING_IN_QUEUE_FOR_MOVE = 4
 
 CLUS_RESTYPE_NAME_VM = "Virtual Machine"
 CLUS_RESTYPE_NAME_VM_CONFIG = "Virtual Machine Configuration"
 
+CLUSREG_NAME_GRP_TYPE = "GroupType"
 CLUSREG_NAME_GRP_STATUS_INFORMATION = 'StatusInformation'
 
 CLUSCTL_GROUP_GET_RO_COMMON_PROPERTIES = 0x3000055
 
+ClusGroupTypeVirtualMachine = 111
+
+CLUSTER_ENUM_NODE = 0x00000001
+CLUSTER_ENUM_RESTYPE = 0x00000002
+CLUSTER_ENUM_RESOURCE = 0x00000004
+CLUSTER_ENUM_GROUP = 0x00000008
+CLUSTER_ENUM_NETWORK = 0x00000010
+CLUSTER_ENUM_NETINTERFACE = 0x00000020
+CLUSTER_ENUM_SHARED_VOLUME_GROUP = 0x20000000
+CLUSTER_ENUM_SHARED_VOLUME_RESOURCE = 0x40000000
+CLUSTER_ENUM_INTERNAL_NETWORK = 0x80000000
+
 # iscsidsc.h
 # ----------
 ISCSI_ANY_INITIATOR_PORT = wintypes.ULONG(-1).value
